<?php

namespace App\Http\Controllers;

use DB;
use Carbon\Carbon;
use App\Models\Driver;
use App\Models\History;
use App\Models\Payment;
use App\Models\Attachment;
use App\Models\DriverNote;
use Illuminate\Http\Request;

class HomeController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function index()
    {
        return view('admin.dashboard');
    }

    public function driver_list()
    {
        $drivers = Driver::paginate(10);
        return view('admin.driver_list',['drivers'=>$drivers]);
    }

    public function add_driver()
    {
        return view('admin.add_driver');
    }

    public function profile_driver($id)
    {
        $driver = Driver::where('id',$id)->first();
        $histories = History::where('driver_id', $id)->where('payment',1)->get();
        $histories_sub = History::where('driver_id', $id)->where('payment', 0)->get();
        // dd($histories_sub);
        $year = Carbon::now()->year;

        $payments = Payment::where('year',$year)->where('driver_id',$driver->id)->get();
        $year = Carbon::now()->year;
        $weeksInYear = Carbon::now()->isoWeeksInYear;
        return view('admin.profile_driver',['weeksInYear'=> $weeksInYear,'histories_sub'=> $histories_sub,'driver'=>$driver,'payments'=>$payments, 'histories'=> $histories]);
    }


    public function save_driver(Request $request)
    {
        $request->validate([
            'image' => 'required|image|mimes:jpeg,png,jpg|max:2048',
        ]);

        $driver = new Driver();
        $driver->fullname = $request->fullname;
        $driver->joiningdate = $request->joiningdate;
        $driver->depot = $request->depot;


        if ($request->hasFile('image')) {
            $imagePath = $request->file('image')->store('riders', 'public'); // Save locally in the 'riders' folder
        }

        $driver->image = $imagePath;
        $driver->save();

        $year = Carbon::now()->year;
        $weeksInYear = Carbon::now()->isoWeeksInYear;

        for ($week = 1; $week <= $weeksInYear; $week++) {
            $payment = new Payment();
            $payment->week = $week;
            $payment->year = $year;
            $payment->driver_id = $driver->id;
            $payment->save();
        }

        session()->flash('success', "Record saved successfully");
        return redirect()->back();
    }

    public function weekdataadd(Request $request)
    {
        $payment = Payment::where('id',$request->id)->first();

        $payment->deposits = $request->deposits;
        $payment->damages = $request->damages;
        $payment->fines = $request->fines;
        $payment->arc = $request->arc;
        $payment->loans = $request->loans;
        $payment->offhire = $request->offhire;
        $payment->week_deficit = $request->week_deficit;
        $payment->old_exit = $request->old_exit;
        $payment->insurance_excess = $request->insurance_excess;
        $payment->status = 1;
        $payment->save();

        session()->flash("success", "Record Saved!");
        return redirect()->back();
    }

    public function submitdriverpayment(Request $request)
    {
        // dd($request->attachments);
        $driver = Driver::where('id',$request->driver_id)->first();
        $ff = $request->field;
        $driver->$ff = $request->values + $driver->$ff;
        $driver->save();

        $history = new History();
        $history->type = $ff;
        $history->amount = $request->values;
        $history->reason = $request->reason;
        $history->driver_id = $driver->id;
        $history->payment = 1;
        $history->save();


        if ($request->has('attachments')) {
        // dd();
            foreach($request->file('attachments') as $image);
            {
                $attachment = new Attachment();
                $attachment->history_id = $history->id;
                $imagePath = $image->store('attachments', 'public'); // Save locally in the 'riders' folder
                $attachment->file = $imagePath;
                $attachment->save();
            }
        }

        session()->flash("success", "Record Saved!");
        return redirect()->back();
    }

    public function submitdriverpaymentrefund(Request $request)
    {
        $driver = Driver::where('id', $request->driver_id)->first();
        $ff = $request->field;
        $driver->$ff = $driver->$ff - $request->values ;
        $driver->save();

        $history = new History();
        $history->type = $ff;
        $history->amount = $request->values;
        $history->reason = $request->reason;
        $history->driver_id = $driver->id;
        $history->payment = 0;
        $history->save();

        if ($request->has('attachments')) {
            // dd();
            foreach ($request->file('attachments') as $image); {
                $attachment = new Attachment();
                $attachment->history_id = $history->id;
                $imagePath = $image->store('attachments', 'public'); // Save locally in the 'riders' folder
                $attachment->file = $imagePath;
                $attachment->save();
            }
        }

        session()->flash("success", "Record Saved!");

        return redirect()->back();
    }

    public function notes_driver($id)
    {
        $driver = Driver::findOrFail($id);

        $histories = History::where('driver_id', $id)->where('payment', 1)->get();
        $histories_sub = History::where('driver_id', $id)->where('payment', 0)->get();
        // dd($histories_sub);
        $year = Carbon::now()->year;

        $payments = Payment::where('year', $year)->where('driver_id', $driver->id)->get();

        $notes = DriverNote::where('driver_id', $id)->get();

        return view('admin.notes_driver',['notes'=> $notes,'driver'=>$driver,'histories'=>$histories,'histories_sub'=>$histories_sub,'payments'=>$payments]);

    }

    public function submitdrivernote(Request $request)
    {
        $note = new DriverNote();
        $note->driver_id = $request->driver_id;
        $note->note = $request->note;

        if ($request->hasFile('file')) {
            $filePath = $request->file('file')->store('notes', 'public'); // Save locally in the 'riders' folder
        }
        $note->file = $filePath ?? null;
        $note->save();

        session()->flash('success', "Note added successfully");
        return redirect()->back();
    }

    public function edit_driver($id)
    {
        $driver = Driver::findOrFail($id);

        return view('admin.edit_driver',['driver'=>$driver]);
    }


    public function submitdriverupdate(Request $request)
    {
        $request->validate([
            'image' => 'image|mimes:jpeg,png,jpg|max:2048',
        ]);

        $driver = Driver::where('id',$request->driver_id)->first();
        $driver->fullname = $request->fullname;
        $driver->joiningdate = $request->joiningdate;
        $driver->depot = $request->depot;


        if ($request->hasFile('image')) {
            $imagePath = $request->file('image')->store('riders', 'public'); // Save locally in the 'riders' folder
            $driver->image = $imagePath;
        }


        $driver->save();

        session()->flash('success', "Record saved successfully");
        return redirect()->back();
    }

}
